package net.es.oscars.rmi.bss;

/**
 * Make sure the graph is populated for an object.  Need to look more at
 * Hibernate to see if there is a way to force load of complete graph with
 * query.
 *
 * @author Evangelos Chaniotakis, David Robertson
 */

import java.util.*;

import org.hibernate.Hibernate;
import net.es.oscars.bss.Reservation;
import net.es.oscars.bss.topology.Layer2Data;
import net.es.oscars.bss.topology.Layer3Data;
import net.es.oscars.bss.topology.MPLSData;
import net.es.oscars.bss.topology.Path;
import net.es.oscars.bss.topology.PathType;
import net.es.oscars.bss.topology.PathElem;
import net.es.oscars.bss.topology.Link;
import net.es.oscars.bss.topology.PathElemParam;

/**
 * BssRmiUtils - Hibernate utils for BSS RMI interface
 */
public class BssRmiUtils {

    public static void initialize(Reservation reservation) {
        Hibernate.initialize(reservation);
        Hibernate.initialize(reservation.getToken());
        Map<String,Path> paths = reservation.getPathMap();
        for (String pathType: paths.keySet()) {
            Path path = paths.get(pathType);
            if (path == null) {
                continue;
            }
            if (path.getLayer2DataSet() != null) {
                Hibernate.initialize(path.getLayer2DataSet());
                Iterator<Layer2Data> l2it = (Iterator<Layer2Data>) path.getLayer2DataSet().iterator();
                while (l2it.hasNext()) {
                    Layer2Data l2data = l2it.next();
                    Hibernate.initialize(l2data);
                }
            }
            if (path.getLayer3DataSet() != null) {
                Hibernate.initialize(path.getLayer3DataSet());
                Iterator<Layer3Data> l3it = (Iterator<Layer3Data>) path.getLayer3DataSet().iterator();
                while (l3it.hasNext()) {
                    Layer3Data l3data = l3it.next();
                    Hibernate.initialize(l3data);
                }
            }
            Hibernate.initialize(path.getMplsDataSet());
            Hibernate.initialize(path.getNextDomain());
            for (PathElem pe : path.getPathElems()) {
                for (PathElemParam pep: pe.getPathElemParams()) {
                    Hibernate.initialize(pep);
                }
                if (pe.getLink() == null) {
                    continue;
                }
                // maybe add more null checks?
                Hibernate.initialize(pe.getLink());
                Hibernate.initialize(pe.getLink().getIpaddrs());
                Hibernate.initialize(pe.getLink().getRemoteLink());
                Hibernate.initialize(pe.getLink().getL2SwitchingCapabilityData());
                Hibernate.initialize(pe.getLink().getPort());
                Hibernate.initialize(pe.getLink().getPort().getNode());
                Hibernate.initialize(pe.getLink().getPort().getNode().getNodeAddress());
                Hibernate.initialize(pe.getLink().getPort().getNode().getDomain());
               // Hibernate.initialize(pe.getLink().getPort().getNode().getDomain().getSite());
            }
        }
    }

}
